#!/usr/bin/perl

($dir, $stage) = @ARGV;

sub get_architecture {

    local($pairs, *left, *right) = @_;
    local(@pairs) = (split(/\s+/, $pairs));
    local($error) = (0);

    foreach $i (0..$#pairs) {

	$pairs[$i] =~ m/(\S+)\-v\-(\S+)/; $a = $1; $b = $2;

	if(($left{$a} && $left{$b}) || ($right{$a} && $right{$b})) {

	    return 1;
	}
	elsif(!$left{$a} && !$left{$b}) {
	    
	    if($right{$b}) {
	    
		$left{$a} = 1;
	    }
	    elsif($right{$a}) {

		$left{$b} = 1;
	    }
	    else {

		local(%orig_left) = (%left);
		local(%orig_right) = (%right); 
		$left{$a} = $right{$b} = 1; $error = &get_architecture(join(" ", @pairs[$i+1..$#pairs]), *left, *right);

		if($error == 1) {
		    
		    %left = %orig_left; %right = %orig_right;
		    $left{$b} = $right{$a} = 1; $error = &get_architecture(join(" ", @pairs[$i+1..$#pairs]), *left, *right);
		}
	    }
	}
	elsif($left{$a}) {

	    $right{$b} = 1;
	}
	else {

	    $right{$a} = 1;
	}
    }

    return $error;
}

@segments = ("HA", "NA", "NP", "NS", "MP", "PB1", "PB2", "PA");

if($stage == 1) {

    for $i (0..$#segments) {

	$seg = "$dir/alignments/$segments[$i]";
        # clk edit: 3/8/10: combine into single command that is run in the background
        # this will also automatically stop processing if a previous command fails
        print "Aligning and running MrBayes on segment fasta file: $seg\n";
        system("(muscle -in $seg\.fa -out $seg\-aln\.fa && " .
               "./get_nexus_file.sh $seg\-aln && " .
               "echo \"execute $seg\-aln\.nex;\" | cat - mb_generic_commands | mb) >/dev/null 2>&1 &");
    }

    exit;
}

for $i (0..$#segments) {

    for $j ($i+1..$#segments) {

        $pair = "$segments[$i]\-v-$segments[$j]";
	if($stage == 2) {
            print "Finding pairwise reassortments: $pair\n";

	    `mkdir $dir/$pair`;

	    `cp $dir/alignments/$segments[$i]\-aln.nex* $dir/$pair`;
	    `rename $segments[$i]\-aln left $dir/$pair/$segments[$i]\-aln*`;

	    `cp $dir/alignments/$segments[$j]\-aln.nex* $dir/$pair`;
	    `rename $segments[$j]\-aln right $dir/$pair/$segments[$j]\-aln*`;

	    `./find_reassortments.pl $dir/$pair 1`;
	}
	
	if($stage == 3) {

	    $report = `cat $dir/$pair/report`; @sets = $report =~ m/\{(.+)\}/g;
	    map {$discovered{$_} .= " $pair";} @sets; 
	}
    }
}

exit if($stage == 2);

print "Processing pairwise runs...\n";
foreach $set (sort {scalar(split(/\s+/, $discovered{$b})) <=> scalar(split(/\s+/, $discovered{$a}))} keys(%discovered)) {

    $evidence = scalar(split(/\s+/, $discovered{$set}))-1;
    if($evidence >= 3) {

	print "$set : $discovered{$set} : "; 
	%left = %right = (); ($error) = &get_architecture($discovered{$set}, *left, *right); 
	@left = sort keys(%left); @right = sort keys(%right); 
	printf "@left | @right %s\n", ($error ? "**" : "");
    }
}

