#!/usr/bin/perl

($dir, $filter, $to_file) = @ARGV;

$out = STDOUT;
open($out, ">$dir/report") if($to_file);

print $out "<<GiRaF Report>>\n\n";
print $out "Known: " if(-e "$dir/implants");
open(IMP, "$dir/implants");
while($line = <IMP>) {

    (@set) = split(/\s+/, $line);
    pop @set if($set[$#set] !~ m/[a-zA-Z]/);

    $implanted{join(" ", sort @set)} = 1;
    foreach $taxa (@set) {

	$moved{$taxa} = 1;
    }

    print $out "{@set} ";
}
close(IMP);
print $out "\n\n" if(-e "$dir/implants");

@label_map = `cat $dir/graph.labels`;
map {($id, @rest) = split; $label_map{$id} = "@rest" if($id ne "");} @label_map;
$labels = `cat $dir/bicliques`;
$total = $perfect = $hybrid = 0;

while($labels =~ m/Label: (\d+)\n\n([^-]+)/g) {

    $id = $1; @set = split(/\s+/, $label_map{$id});
    $cliques = $2; 

    $p = 1; @cliques = ($cliques =~ m/.*\n.*\n(\S+ \S+)\n\n/g);
    map { ($a, $b) = split; $p *= 1-$a*$b; } @cliques;
    $conf = 1-$p;

    next if($filter && @cliques == 1);
    print $out "Candidate = ID: $id, Conf: $conf, Taxa: {@set}\n"; 	
	
    $moved = 1;
    foreach $taxa (@set) {
	
	$moved = 0 if(!$moved{$taxa});
	$found{$taxa} = 1 if($moved{$taxa});
	$marked{$taxa} = 1;
    }

    if($implanted{join(" ", sort @set)}) {
	    
	print $out "Status: Perfect Match\n";
	$perfect++;
    }
    elsif($moved) {
	    
	print $out "Status: Hybrid Match\n";
	$hybrid++;
    }
		
    $total++;
}

$marked_implant = 0;
foreach $set (keys(%implanted)) {

    $marked = 1;
    foreach $taxa (split(/\s+/, $set)) {

	$marked = 0 if(!$marked{$taxa});
    }

    $marked_implant++ if($marked);
}

printf $out "\nTaxa = Moved: %d, Marked: %d, Found: %d\n", scalar(keys(%moved)), scalar(keys(%marked)), scalar(keys(%found)); 
printf $out "Sets = Moved: %d, Marked: $total, Perfect: $perfect, Hybrid: $hybrid, Identified: $marked_implant\n", scalar(keys(%implanted));
