#!/usr/bin/perl

sub get_parens {

    local($text, *parens) = @_;

    local(@starts) = (());
    for $i (0..length($text)-1) {

	push @starts, $i if(substr($text, $i, 1) eq '(');
	if(substr($text, $i, 1) eq ')') {

	    $start = pop @starts;
	    push @parens, substr($text, $start, $i-$start+1);
	}
    } 
}

sub update_splits {

    local($tree, $index, $size, *splits) = @_;

    local(@parens) = (());
    &get_parens($tree, *parens);

    foreach $set (@parens) {

	local(@set, @split_string, $split_string);
	$set =~ s/\)|\(//g;
	@set = split(/,/, $set); 
	@split_string = ('*')x$size;
	map { $split_string[$_-1] = '.' } @set;
	$split_string = join("", @split_string);
	$splits{$split_string} .= " $index";
    }
}

$branch_len = 8;

sub update_distances {

    local($tree, $size, *dist) = @_;

    local($total) = (0);
    local($max, $child) = ($size, ""); local(%tree_dist) = (()); local(%all_dist) = (()); local(@parents) = (());
    foreach $i (0..length($tree)-1) {

	push @parents, $size++ if(substr($tree, $i, 1) eq "(");
	$child = pop @parents if(substr($tree, $i, 1) eq ")");

	if(substr($tree, $i, 1) eq ":") {

	    if(substr($tree, $i-1, 1) ne ")") {
		substr($tree, 0, $i+1) =~ m/(\d+):\Z/;
		$child = $1-1;		
	    }
	    
	    $tree_dist{$child}{$parents[$#parents]} = $all_dist{$child}{$parents[$#parents]} = 0+substr($tree, $i+1, $branch_len);
	    $tree_dist{$parents[$#parents]}{$child} = $all_dist{$parents[$#parents]}{$child} = 0+substr($tree, $i+1, $branch_len);

	    $total += 0+substr($tree, $i+1, $branch_len);
	}
    } 

    for $i (0..$max-1) {

	@neighbors = keys(%{$tree_dist{$i}});

	while(@neighbors != 0) {

	    $curr = pop @neighbors;
	    foreach $adjacent (keys(%{$tree_dist{$curr}})) {

		if($adjacent != $i && $all_dist{$i}{$adjacent} == 0) {
		    $all_dist{$i}{$adjacent} = $all_dist{$adjacent}{$i} = $all_dist{$i}{$curr}+$all_dist{$curr}{$adjacent};
		    unshift @neighbors, $adjacent;
		}
	    }
	}
    }

    for $i (0..$max-1) {

	for $j ($i+1..$max-1) {

	    $dist[$i][$j] .= (($all_dist{$i}{$j}/$total)." ");
	}
    }
}

sub read_file {

    local($file, $burnin, $process_distances, *left_splits, *right_splits) = @_;

    open(TREES, "$file\.nex\.run1.t");
    $data1 = join("", <TREES>);
    close(TREES);
    open(TREES, "$file\.nex\.run2.t");
    $data2 = join("", <TREES>);
    close(TREES);

    $data1 =~ m/begin trees;.*\n.*translate.*\n([^;]+);/;
    $names = $1; @names = split(/,\n/, $names); 

    %name2id = map { ($gap, $id, $name) = split(/\s+/, $_); $name => $id } @names;
    %id2name = map { ($gap, $id, $name) = split(/\s+/, $_); $id => $name } @names;
    @taxa_names = sort keys %name2id; 

    @trees1 = $data1 =~ m/tree [^\(]+(\(.*);/g;
    @trees2 = $data2 =~ m/tree [^\(]+(\(.*);/g;

    @trees = (@trees1[$burnin..$#trees1], @trees2[$burnin..$#trees2]);
    %split2trees = (); $num_taxa = scalar @taxa_names; 
    

    for $i (0..$#trees) {

	$tree = $trees[$i]; $tree =~ s/:[^,)(\s]+//g; 
	&update_splits($tree, $i, $num_taxa, *split2trees);
    }

    if($process_distances > 0) {

	@dist = ();
	for $i (0..$#trees) {

	    &update_distances($trees[$i], $num_taxa, *dist);
	}

	open(DIST, ">$file\_dist");
	for $i (0..$num_taxa-1) {

	    for $j ($i+1..$num_taxa-1) {
		
		printf DIST "%s %s %s\n", $taxa_names[$i], $taxa_names[$j], 
		($dist[$name2id{$taxa_names[$i]}-1][$name2id{$taxa_names[$j]}-1] eq "" ? 
		 $dist[$name2id{$taxa_names[$j]}-1][$name2id{$taxa_names[$i]}-1] : 
		 $dist[$name2id{$taxa_names[$i]}-1][$name2id{$taxa_names[$j]}-1]);   
	    }
	}
	close(DIST);
    }

    open(OUT, ">$file\_trees");
    open(TRANS, ">$file\_splits");

    $split_no = 0;
    foreach $split (keys(%split2trees)) {

	next if($split2trees{$split} eq "");

	local(@left, @right) = ((), ());
	for $i (0..length($split)-1) {

	    push @left, $id2name{$i+1} if(substr($split, $i, 1) eq '.');
	    push @right, $id2name{$i+1} if(substr($split, $i, 1) eq '*');
	}

	next if (@left == 1 || @right == 1);

	$rev_split = $split; $rev_split =~ tr/.*/*./; $list_of_trees = $split2trees{$split}.$split2trees{$rev_split};
	$split2trees{$split} = $split2trees{$rev_split} = "";
	@list_of_trees = split(/\s+/, $list_of_trees); 

	next if(@list_of_trees/@trees < 0.05);

	@left = sort @left; @right = sort @right;
	push @left_splits, [@left]; push @right_splits, [@right]; 
	print OUT "$split_no $#list_of_trees$list_of_trees\n";
	print TRANS "$split_no \{@left\} \{@right\}\n";

	$split_no++;
    }

    close(OUT);
    close(TRANS);

    $num_of_trees = @trees;
    `echo \">> $num_of_trees $split_no\" | cat - $file\_trees > $file\_tmp`;
    `mv $file\_tmp $file\_trees`;
}

sub empty_intersection {

    local(*set1, *set2) = @_;

    %set2 = map { $_ => 1} @set2;
    foreach $value (@set1) {

	return 0 if($set2{$value});
    }

    return 1;
}

($dir, $burnin, $process_distances) = @ARGV;

@left_splits1 = @left_splits2 = ();
@right_splits1 = @right_splits2 = ();
&read_file("$dir/left", $burnin, $process_distances, *left_splits1, *right_splits1);
&read_file("$dir/right", $burnin, $process_distances, *left_splits2, *right_splits2);

open(GRAPH, ">$dir/graph");
for($i = 0; $i < @left_splits1; $i++) {

    for($j = 0; $j < @left_splits2; $j++) {

	print GRAPH "$i $j\n" if(!empty_intersection($left_splits1[$i], $left_splits2[$j]) &&
				 !empty_intersection($right_splits1[$i], $left_splits2[$j]) &&
				 !empty_intersection($left_splits1[$i], $right_splits2[$j]) &&
				 !empty_intersection($right_splits1[$i], $right_splits2[$j]));
    }
}
close(GRAPH);
