#!/usr/bin/perl
use POSIX qw(floor);

sub avg {

    local(*data) = @_;
    local($sum) = (0);
    map {$sum += $_} @data;

    return $sum/@data;
}

sub sd {

    local(*data) = @_;
    local($avg) = (&avg(*data));
    local($sum) = (0);
    map {$sum += ($_-$avg)*($_-$avg)} @data;
    $sum = 1 if($sum == 0);

    return sqrt($sum/(@data-1));
}

sub get_pvalue {

    local($left_dist, $right_dist) = @_;
    local(@left_dist) = (split(/\s+/, $left_dist));
    local(@right_dist) = (split(/\s+/, $right_dist));

    print "Error: Lengths of distance vectors disagree.\n" if(@left_dist != @right_dist);

    $z_score = (&avg(*left_dist)-&avg(*right_dist))/&sd(*right_dist);
    $corr_z_score = sqrt($z_score*$z_score); $corr_z_score *= 2;
    $log_p = `./normal_invcdf $corr_z_score`; chomp $log_p;

    return (($z_score > 0 ? 1 : 0), $log_p);
}

($dir) = @ARGV;

open(LEFT_DIST, "$dir/left_dist");
open(RIGHT_DIST, "$dir/right_dist");
open(OUT, ">$dir/pair_test_results");

while(<LEFT_DIST>) {
    
    $_ =~ m/(\S+) (\S+) (.*)/; 
    $taxa1 = $1; $taxa2 = $2; $left_dist = $3;
    $line = <RIGHT_DIST>;
    $line =~ m/(\S+) (\S+) (.*)/;
    print "Error: Dist files not in sync for $taxa1 $taxa2 $1 $2\n" if($taxa1 ne $1 || $taxa2 ne $2);
    $right_dist = $3;
    ($is_greater, $log_pvalue) = &get_pvalue($left_dist, $right_dist);
    print OUT "$taxa1 $taxa2 $is_greater $log_pvalue\n";
} 

close(LEFT_DIST);
close(RIGHT_DIST);
