#!/bin/sh

# make sure the given command exists
check_exists () 
{
    if which $1 > /dev/null ; then
        return 0
    fi
    echo "error: GIRAF requires $1 ($2) which could not be found." > /dev/stderr
    return 3
}

#
# Check that the external dependencies exist to run it
#
if check_exists muscle "http://www.drive5.com/muscle/" && \
   check_exists mb "MrBayes http://mrbayes.csit.fsu.edu/" && \
   check_exists perl "Perl http://www.perl.org/" && \
   check_exists grep "unix grep command" && \
   check_exists wc "unix word count command" && \
   check_exists cat "unix cat command" && \
   check_exists cp "unix copy command" && \
   check_exists mv "unix move command" && \
   check_exists rename "unix rename command" && \
   check_exists echo "unix echo command" ; then
    echo "The programs you need to run seem to be installed. Great!" > /dev/stderr
else
    echo "fatal: missing one or more external dependencies!" > /dev/stderr
    echo "fatal: GIRAF will not run." > /dev/stderr
fi

#
# Try to build get_max_biclique if the user wants
#
cat <<EOT

Part of GIRAF must be compiled. If the downloaded get_max_biclique
works for you, you don't have to recompile it. Otherwise, you should.

EOT

read -re -p "Do you want to compile the core of GIRAF? [yN] " YN
if [ "$YN" == "y" -o "$YN" == "Y" ] ; then
    echo "Trying to build get_max_bicliques..."
    if [ -e bin/get_max_bicliques ] ; then
        mv bin/get_max_bicliques bin/get_max_bicliques.original
    fi
    if [ -e src/get_max_bicliques ] ; then
        mv src/get_max_bicliques src/get_max_bicliques.original
    fi
    if [ -e bin/normal_invcdf ] ; then
        mv bin/normal_invcdf bin/normal_invcdf.original
    fi
    if [ -e src/normal_invcdf ] ; then
        mv src/normal_invcdf src/normal_invcdf.original
    fi
    if [ -e bin/binomial_invcdf ] ; then
        mv bin/binomial_invcdf bin/binomial_invcdf.original
    fi
    if [ -e src/binomial_invcdf ] ; then
        mv src/binomial_invcdf src/binomial_invcdf.original
    fi
    (cd src; make)
fi

